package com.ichaoj.ams.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import com.ichaoj.ams.entity.AmsAirdropTask;
import com.ichaoj.ams.request.airdrop.PageProjectRequest;
import com.ichaoj.ams.request.task.CreateAirdropTask;
import com.ichaoj.ams.request.task.PageTaskRequest;
import com.ichaoj.ams.request.task.UpdateAirdropTask;
import com.ichaoj.ams.request.task.UpdateTaskStatusRequest;
import com.ichaoj.ams.response.airdrop.AirdropProjectResponse;
import com.ichaoj.ams.response.task.TaskResponse;
import com.ichaoj.ams.script.IScript;
import com.ichaoj.ams.script.ScriptContext;
import com.ichaoj.ams.script.annotation.Script;
import com.ichaoj.ams.script.annotation.ScriptParam;
import com.ichaoj.ams.script.model.AirdropParam;
import com.ichaoj.ams.service.IAmsAirdropTaskService;
import com.ichaoj.common.annotation.AuthResource;
import com.ichaoj.common.exception.ErrorServiceException;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.context.annotation.Lazy;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 空投项目任务 前端控制器
 * </p>
 *
 * @author cjwen
 * @since 2023-04-25
 */
@RestController
@RequestMapping("/airdrop-task")
@Tag(name = "任务模块")
public class AirdropTaskController {

    @Resource
    private IAmsAirdropTaskService taskService;

    @PostMapping
    @Operation(summary = "创建空投任务")
    @AuthResource(true)
    public PublicResult<Object> createAirdropTask(@RequestBody CreateAirdropTask createAirdropTask) {
        taskService.save(BeanUtil.copyProperties(createAirdropTask, AmsAirdropTask.class));
        return PublicResult.success();
    }

    @PutMapping
    @Operation(summary = "修改空投任务")
    @AuthResource(true)
    public PublicResult<Object> updateAirdropTask(@RequestBody @Validated UpdateAirdropTask updateAirdropTask) {
        taskService.updateById(BeanUtil.copyProperties(updateAirdropTask, AmsAirdropTask.class));
        return PublicResult.success();
    }

    @PutMapping("update-status")
    @Operation(summary = "修改空投任务状态")
    @AuthResource
    public PublicResult<Object> updateAirdropTaskStatus(@RequestBody @Validated UpdateTaskStatusRequest statusRequest) {
        AmsAirdropTask task = taskService.getById(statusRequest.getAmsTaskId());
        if (task != null) {
            task.setTaskStatus(statusRequest.getTaskStatus());
            taskService.updateById(task);
        } else {
            throw new ErrorServiceException("Task Error");
        }
        return PublicResult.success();
    }

    @DeleteMapping("/{taskId}")
    @Operation(summary = "删除空投任务")
    @AuthResource(true)
    public PublicResult<Object> deleteAirdropTask(@PathVariable String taskId) {
        taskService.removeById(taskId);
        return PublicResult.success();
    }

    @PostMapping("/page")
    @Operation(summary = "分页查询空投任务")
    @AuthResource
    public PublicResult<PublicPage<TaskResponse>> pageTask(@RequestBody PageTaskRequest taskRequest) {
        PublicPage<TaskResponse> result = taskService.pageAirdropProject(taskRequest);
        return PublicResult.success(result);
    }

    @GetMapping("/list")
    @Operation(summary = "查询任务列表")
    @AuthResource
    public PublicResult<List<TaskResponse>> listTask() {
        List<TaskResponse> result = taskService.listTask();
        return PublicResult.success(result);
    }

    @GetMapping("/params/{task-code}")
    @AuthResource
    @Operation(summary = "查询任务参数")
    public PublicResult<List<AirdropParam>> getAirdropParam(@PathVariable("task-code") String taskCode) {
        IScript script = ScriptContext.getScriptByCode(taskCode);
        Script annotation = script.getClass().getAnnotation(Script.class);
        ScriptParam[] params = annotation.params();
        if (params != null && params.length > 0) {
            List<AirdropParam> airdropParams = new ArrayList<>();
            for (ScriptParam param : params) {
                AirdropParam airdropParam = new AirdropParam();
                airdropParam.setName(param.name());
                airdropParam.setNote(param.note());
                airdropParam.setValue(param.defaultValue());
                airdropParams.add(airdropParam);
            }
            return PublicResult.success(airdropParams);
        }
        return PublicResult.success(ListUtil.empty());
    }
}
