package com.ichaoj.ams.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ichaoj.ams.common.bean.Wallet;
import com.ichaoj.ams.common.util.WalletUtil;
import com.ichaoj.ams.common.util.ZipUtil;
import com.ichaoj.ams.entity.AmsAddressAccount;
import com.ichaoj.ams.mapper.AmsAddressAccountMapper;
import com.ichaoj.ams.request.address.BatchAddressRequest;
import com.ichaoj.ams.request.address.PageAddressRequest;
import com.ichaoj.ams.request.address.PremiumAddressRequest;
import com.ichaoj.ams.response.address.AddressResponse;
import com.ichaoj.ams.response.address.CountAddressResponse;
import com.ichaoj.ams.script.model.AirdropWallet;
import com.ichaoj.ams.service.IAmsAddressAccountService;
import com.ichaoj.ams.service.IAmsTradeRecordService;
import com.ichaoj.common.exception.ErrorServiceException;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicUserInfo;
import com.ichaoj.mybatis.service.SuperWhaleServiceImpl;
import com.ichaoj.web.context.SuperWhaleContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.ichaoj.ams.constant.AmsConstant.WALLET_FILE_PATH;

/**
 * <p>
 * 地址管理 服务实现类
 * </p>
 *
 * @author cjwen
 * @since 2023-05-06
 */
@Service
@Slf4j
public class AmsAddressAccountServiceImpl extends SuperWhaleServiceImpl<AmsAddressAccountMapper, AmsAddressAccount> implements IAmsAddressAccountService {

    @Resource
    private IAmsTradeRecordService tradeRecordService;

    @Override
    public List<String> batchCreateAddress(BatchAddressRequest batchAddressRequest) {
        String userId = SuperWhaleContext.getContext(PublicUserInfo.class).getUserId();
        List<AmsAddressAccount> addresses = this.getByGroupNameAndUserId(batchAddressRequest.getGroupName(), userId);
        if (CollectionUtil.isNotEmpty(addresses)) {
            throw new ErrorServiceException("Please change the groupName because you have created");
        }
        long start = System.currentTimeMillis();
        List<Wallet> wallets = WalletUtil.batchCreateWallet(
                userId, batchAddressRequest.getNumWallet(),
                batchAddressRequest.getPassword(),
                batchAddressRequest.getGroupName());
        long end = System.currentTimeMillis();
        log.info("本次共创建{}个钱包地址，耗时 {} 秒", wallets.size(), (end - start) / 1000);
        wallets.forEach(wallet -> {
            try {
                String keystore = wallet.getKeystore();
                // 将生成的地址存入数据库
                AmsAddressAccount addressAccount = getAddressCountInstance(
                        userId, wallet.getAddress(), 1,
                        batchAddressRequest.getChainId()
                        , batchAddressRequest.getGroupName());
                addressAccount.setKeystore(keystore);
                this.save(addressAccount);
            } catch (Exception e) {
                wallets.forEach(w -> {
                    File file = new File(WALLET_FILE_PATH + userId + File.separator +
                            batchAddressRequest.getGroupName() + File.separator + w.getKeystore());
                    if (file.exists()) {
                        file.delete();
                    }
                });
                throw new ErrorServiceException(e.getMessage());
            }
        });
        return wallets.stream().map(Wallet::getAddress).collect(Collectors.toList());
    }

    @Override
    public List<AmsAddressAccount> getByGroupNameAndUserId(String groupName, String userId) {
        LambdaQueryWrapper<AmsAddressAccount> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(AmsAddressAccount::getUserId, userId)
                .eq(AmsAddressAccount::getGroupName, groupName);
        return this.list(wrapper);
    }

    @Override
    public PublicPage<AddressResponse> pageAddress(PageAddressRequest pageAddressRequest) {
        String userId = SuperWhaleContext.getContext(PublicUserInfo.class).getUserId();
        Page<AddressResponse> result = this.baseMapper.pageAddress(this.buildPageObj(pageAddressRequest), pageAddressRequest, userId);
        return this.convertPublicPage(result, resp -> BeanUtil.copyProperties(resp, AddressResponse.class));
    }

    @Override
    public List<String> queryGroupList(int addressType) {
        LambdaQueryWrapper<AmsAddressAccount> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(AmsAddressAccount::getUserId, SuperWhaleContext.getContext(PublicUserInfo.class).getUserId())
                .eq(addressType >= 0, AmsAddressAccount::getAddressType, addressType)
                .groupBy(AmsAddressAccount::getGroupName);
        return this.list(wrapper).stream().map(AmsAddressAccount::getGroupName).distinct().collect(Collectors.toList());
    }

    @Override
    public AmsAddressAccount getByAddress(String address) {
        LambdaQueryWrapper<AmsAddressAccount> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(AmsAddressAccount::getAddress, address);
        return this.getOne(wrapper);
    }

    @Override
    public List<CountAddressResponse> countBatchCount(String userId) {
        if (StrUtil.isBlank(userId)) {
            userId = SuperWhaleContext.getContext(PublicUserInfo.class).getUserId();
        }
        return this.baseMapper.countBatchCount(userId);
    }

    @Override
    public List<String> queryBatchAddress(int addressType) {
        LambdaQueryWrapper<AmsAddressAccount> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(AmsAddressAccount::getUserId, SuperWhaleContext.getContext(PublicUserInfo.class).getUserId())
                .eq(addressType >= 0, AmsAddressAccount::getAddressType, addressType)
                .groupBy(AmsAddressAccount::getAddress);
        return this.list(wrapper).stream().map(AmsAddressAccount::getAddress).distinct().collect(Collectors.toList());
    }

    @Override
    public List<AirdropWallet> getAirdropAddressByAccount(List<AmsAddressAccount> accountList, String userId, String groupName, String password) {
        return accountList
                .stream()
                .map(account -> WalletUtil.getWalletByKeystore(account.getKeystore(), userId, groupName, password))
                .collect(Collectors.toList());
    }

    /**
     * 添加精品号
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addPremiumAddress(List<PremiumAddressRequest> requestList) {
        String userId = SuperWhaleContext.getContext(PublicUserInfo.class).getUserId();
        List<AmsAddressAccount> accounts = new ArrayList<>();
        for (PremiumAddressRequest request : requestList) {
            verifyDuplicate(requestList.stream().map(PremiumAddressRequest::getGroupName).collect(Collectors.toList()));
            List<AmsAddressAccount> addresses = this.getByGroupNameAndUserId(request.getGroupName(), userId);
            if (CollectionUtil.isNotEmpty(addresses)) {
                throw new ErrorServiceException("你已经创建了" + request.getGroupName() + "精品号,请更换地址组名称重试！");
            }
            AmsAddressAccount account = getAddressCountInstance(userId,
                    request.getAddress(),
                    0,
                    request.getChainId(),
                    request.getGroupName());
            accounts.add(account);
        }
        this.saveBatch(accounts);
    }

    /**
     * 验证 List<PremiumAddressRequest> 列表是否有重复对象PremiumAddressRequest，如有重复，返回重复对象的下标
     */
    public void verifyDuplicate(List<String> list) {
        List<Integer> duplicates = new ArrayList<>();
        // 使用Set记录已经出现过的对象
        Set<String> set = new HashSet<>();
        for (int i = 0; i < list.size(); i++) {
            String request = list.get(i);
            // 如果set中已经包含obj,则表示obj重复
            if (set.contains(request)) {
                duplicates.add(i + 1);
            } else {
                set.add(request);
            }
        }
        String collect = duplicates.stream().map(String::valueOf).collect(Collectors.joining(StrUtil.COMMA));
        if (StrUtil.isNotBlank(collect)) {
            throw new ErrorServiceException("你创建的第" + collect + "个精品号是重复的,请更换地址组名称重试！");
        }
    }

    @Override
    public void batchKeystore(String groupName, HttpServletResponse response) {
        String userId = SuperWhaleContext.getContext(PublicUserInfo.class).getUserId();
        List<AmsAddressAccount> addresses = this.getByGroupNameAndUserId(groupName, userId);
        if (CollectionUtil.isEmpty(addresses)) {
            throw new ErrorServiceException("地址组名称不正确");
        }

        if (addresses.get(0).getAddressType() != 1) {
            throw new ErrorServiceException(groupName + "为非批量号地址组，不允许下载");
        }

        ZipUtil.createZip(groupName, response, userId);
    }

    private AmsAddressAccount getAddressCountInstance(
            String userId, String address,
            Integer addressType, String chainId, String groupName) {
        AmsAddressAccount account = new AmsAddressAccount();
        account.setAddress(address);
        account.setAddressType(addressType);
        account.setCreateTime(LocalDateTime.now());
        account.setUserId(userId);
        account.setChainId(chainId);
        account.setGroupName(groupName);
        return account;
    }
}
